import os
import subprocess
from typing import Optional
from streamlit_hub.models.App import RepoApp


class RepoAccess:
    base_path = os.path.join(os.path.expanduser("~"), ".streamlit-hub", "repos")
    repo_name: str
    repo_path: str

    def __init__(self, app: RepoApp):
        if not os.path.exists(self.base_path):
            os.makedirs(self.base_path)
        self.app = app
        self.repo_name = app.repo_url.split("/")[-1].replace(".git", "")
        self.repo_path = os.path.join(self.base_path, self.repo_name)
        RepoAccess.clone_repo(app.repo_url, self.repo_path)
        self.checkout_branch(app.branch)

    @staticmethod
    def clone_repo(url, destination):
        if not RepoAccess._is_git_initialized(destination):
            os.makedirs(destination)
            # Clone the repository if it doesn't exist
            subprocess.run(["git", "clone", url, destination])

    def checkout_branch(self, branch: str):
        os.chdir(self.repo_path)
        subprocess.run(["git", "checkout", branch])

    def pull(self, branch: Optional[str] = None):
        if branch is not None:
            self.checkout_branch(branch)
        os.chdir(self.repo_path)
        subprocess.run(["git", "pull"])

    @staticmethod
    def _is_git_initialized(directory):
        return os.path.exists(os.path.join(directory, ".git"))



import os
import subprocess
import logging
import threading
import time
from typing import List, Optional
from streamlit_hub.access.registered_apps_access import AppAccess
from streamlit_hub.access.repo_access import RepoAccess
from streamlit_hub.models.App import App, LocalApp, RepoApp

from streamlit_hub.models.RunningProcess import LocalProcess

logger = logging.getLogger(__name__)


class Manager:
    registered_apps: List[App] = []
    app_access = AppAccess()
    occupied_ports: set[int] = set()

    def __init__(self) -> None:
        self.registered_apps = self.app_access.get_list()
        for app in self.registered_apps:
            if app.run_by_default:
                try:
                    self.start_app(app)
                except:
                    logger.error(f"couldnt start {app.name} at startup")
        check_thread = threading.Thread(target=self._clean_closed_processes, args=(), daemon=True)
        check_thread.start()

    def start_app(self, app: App):
        logger.info(f"Starting {app}")
        port = int(app.desired_port) if app.desired_port is not None else self._find_next_port()
        if type(app) is LocalApp:
            path = app.path
        elif type(app) is RepoApp:
            if not app.local_path:
                app.local_path = os.path.join(RepoAccess(app).repo_path, app.streamlit_entry_point_in_repo)
            path = app.local_path
        else:
            logger.error("We can't process other type of app than local apps at the moment")
            return None
        process = subprocess.Popen(
            ["python", "-m", "streamlit", "run", path, "--server.port", str(port)], start_new_session=True
        )
        self.occupied_ports.add(port)
        app.running_process = LocalProcess("", port, process)
        return process

    def register_app(self, app: App) -> Optional[str]:
        if any(map(lambda x: x.name == app.name, self.registered_apps)):
            return "An app with the same name was already added"
        if type(app) is RepoApp:
            access = RepoAccess(app)
            app.local_path = os.path.join(access.repo_path, app.streamlit_entry_point_in_repo)
        self.registered_apps.append(app)
        self.app_access.persist_list(self.registered_apps)

    def toggle_run_default_app(self, app: App, run_by_default: bool) -> Optional[str]:
        app.run_by_default = run_by_default
        self.app_access.persist_list(self.registered_apps)

    def unregister_app(self, app: App) -> Optional[str]:
        self.registered_apps = list(filter(lambda matching_app: app.name != matching_app.name, self.registered_apps))
        self.app_access.persist_list(self.registered_apps)

    def stop_app(self, app: App) -> subprocess.Popen:
        logger.info(f"Stopping {app.name}")
        if app.running_process is None:
            logger.info(f"{app.name} was alreay stopped")
            return
        local_process = app.running_process
        local_process: LocalProcess
        process = local_process.process
        try:
            process.terminate()
            process.wait(timeout=5)  # Wait for the process to finish
            logger.info(f"{app.name} was stopped")
        except subprocess.TimeoutExpired:
            process.kill()  # If terminate() doesn't work, forcefully kill the process
            logger.info(f"{app.name} was forced stopped")
        if local_process.port in self.occupied_ports:
            self.occupied_ports.remove(local_process.port)
        app.running_process = None

    def start_all(self):
        # Start Streamlit processes for each app
        for app in self.registered_apps:
            self.start_app(app)

    def end_all(self):
        for app in self.registered_apps:
            self.stop_app(app)

    def refresh_app(self, app):
        if type(app) is RepoApp:
            access = RepoAccess(app)
            access.pull()

    def _find_next_port(self) -> int:
        for potential in range(8503, 8550):
            if potential in self.occupied_ports:
                continue
            if any(map(lambda x: x.desired_port == potential, self.registered_apps)):
                continue
            return potential
        raise Exception("No new port found")

    def _clean_closed_processes(self):
        while True:
            for app in self.registered_apps:
                if (
                    app.running_process
                    and type(app.running_process) is LocalProcess
                    and app.running_process.process.poll() is not None
                ):
                    logger.warn(f"Process of app '{app.name}' has finished without commanding its closure.")
                    port = app.running_process.port
                    if port in self.occupied_ports:
                        self.occupied_ports.remove(port)
                    app.running_process = None

            time.sleep(1)  # Check every second



import pytest

from streamlit_hub.skeleton import fib, main

__author__ = "Guillaume MÃ¶sching"
__copyright__ = "Guillaume MÃ¶sching"
__license__ = "MIT"


def test_fib():
    """API Tests"""
    assert fib(1) == 1
    assert fib(2) == 1
    assert fib(7) == 13
    with pytest.raises(AssertionError):
        fib(-10)


def test_main(capsys):
    """CLI Tests"""
    # capsys is a pytest fixture that allows asserts against stdout/stderr
    # https://docs.pytest.org/en/stable/capture.html
    main(["7"])
    captured = capsys.readouterr()
    assert "The 7-th Fibonacci number is 13" in captured.out



import subprocess
import dataclasses


@dataclasses.dataclass
class RunningProcess:
    link: str


@dataclasses.dataclass
class LocalProcess(RunningProcess):
    port: str
    process: subprocess.Popen



from streamlit_hub.manager.manager import Manager
import streamlit as st

from streamlit_hub.models.App import LocalApp, RepoApp


def show_status(manager: Manager):
    col1, col2, col3 = st.columns([1, 1, 1])
    col1.write("**App Name**")
    col2.write("**App Details**")
    col3.write("**Actions**")
    for app in manager.registered_apps:
        st.markdown("""---""")
        (
            col1,
            col2,
            col3,
        ) = st.columns([0.2, 1, 1])
        col1.write(f"**{app.name}**")
        if type(app) is LocalApp:
            col2.write(app.path)
        if type(app) is RepoApp:
            col2.write(f"**url:** {app.repo_url}, **branch:** {app.branch}")
        if app.running_process is None:
            col3.button("Start", on_click=manager.start_app, args=(app,), key=("start", app))
        else:
            col3.button("Stop", on_click=manager.stop_app, args=(app,), key=("start", app))

        col3.checkbox(
            "Run at startup",
            value=app.run_by_default,
            on_change=manager.toggle_run_default_app,
            args=(app, not app.run_by_default),
            key=("toggle_default", app),
        )
        if type(app) is RepoApp:
            col3.button("Pull Latest", on_click=manager.refresh_app, args=(app,), key=("refresh", app))

        col3.button("Delete", on_click=manager.unregister_app, args=(app,), key=("Delete", app))
    st.markdown("""---""")



import streamlit as st

from streamlit_hub.manager.manager import Manager
from streamlit_hub.models.App import LocalApp, RepoApp


def local_source(manager: Manager, app_name: str, run_by_default: bool):
    app_path = st.text_input("App Path:")

    if st.button("Register"):
        if app_name and app_path:
            ret = manager.register_app(LocalApp(name=app_name, run_by_default=run_by_default, path=app_path))
            if ret:
                st.error(ret)
            else:
                st.success(f"App {app_name} registered successfully!")
        else:
            st.error("Please provide both App Name and App Path.")


def repo_source(manager: Manager, app_name: str, run_by_default: bool):
    repo_url = st.text_input("Repo URL:")
    repo_branch = st.text_input("Repo Branch:")
    path_in_repo = st.text_input("Path of streamlit entrypoint in Repo:")

    if st.button("Register"):
        if app_name and repo_url and repo_branch and path_in_repo:
            ret = manager.register_app(
                RepoApp(
                    name=app_name,
                    run_by_default=run_by_default,
                    repo_url=repo_url,
                    branch=repo_branch,
                    streamlit_entry_point_in_repo=path_in_repo,
                )
            )
            if ret:
                st.error(ret)
            else:
                st.success(f"App {app_name} registered successfully!")
        else:
            st.error("Please provide both App Name, Repo URL, Report branch, and Path in Repo")


def show_register(manager: Manager):
    st.header("Register New App")

    app_name = st.text_input("App Name:")
    run_by_default = st.checkbox("Run Application at startup")

    source = st.radio("Source of application", ["Local Directory", "Git Repo"])

    if source == "Local Directory":
        local_source(manager, app_name, run_by_default)
    else:
        repo_source(manager, app_name, run_by_default)



import os
import sys

# from streamlit.web import cli as stcli
import logging
import streamlit_hub.client
import runpy

logger = logging.getLogger(__name__)


def main():
    streamlit_script_path = os.path.join(os.path.dirname(streamlit_hub.client.__file__), "app.py")
    sys.argv = ["streamlit", "run", streamlit_script_path, "--theme.base", "dark", "--theme.primaryColor", "orange"]
    runpy.run_module("streamlit", run_name="__main__")


if __name__ == "__main__":
    main()



import json
import os
from typing import List
from streamlit_hub.models.App import App, LocalApp, RepoApp


class AppAccess:
    persisited_base_path = os.path.join(os.path.expanduser("~"), ".streamlit-hub")
    persisited_registered_apps_path = os.path.join(persisited_base_path, "registered_apps.json")

    def __init__(self) -> None:
        if not os.path.exists(self.persisited_base_path):
            os.makedirs(self.persisited_base_path)
        if not os.path.exists(self.persisited_registered_apps_path):
            with open(self.persisited_registered_apps_path, "w") as file:
                json.dump([], file)
        pass

    def get_list(self) -> List[App]:
        ret = []
        with open(self.persisited_registered_apps_path, "r") as file:
            db = json.load(file)
            for app in db:
                if "branch" in app and "streamlit_entry_point_in_repo" in app and "repo_url" in app:
                    ret.append(
                        RepoApp(
                            app["name"],
                            run_by_default=app["default_on"] if "default_on" in app else False,
                            repo_url=app["repo_url"],
                            branch=app["branch"],
                            streamlit_entry_point_in_repo=app["streamlit_entry_point_in_repo"],
                            desired_port=app["desired_port"] if "desired_port" in app else None,
                        )
                    )
                elif "path" in app:
                    ret.append(
                        LocalApp(
                            app["name"],
                            run_by_default=app["default_on"] if "default_on" in app else False,
                            path=app["path"],
                            desired_port=app["desired_port"] if "desired_port" in app else None,
                        )
                    )
                else:
                    raise Exception(f"Could not parse {app} into an App")
        return ret

    def persist_list(self, apps: List[App]):
        encoded = []
        for app in apps:
            new = {"name": app.name, "default_on": app.run_by_default}
            if app.desired_port:
                new["desired_port"] = app.desired_port
            if type(app) is LocalApp:
                new["path"] = app.path
            if type(app) is RepoApp:
                new["branch"] = app.branch
                new["streamlit_entry_point_in_repo"] = app.streamlit_entry_point_in_repo
                new["repo_url"] = app.repo_url
            encoded.append(new)
        with open(self.persisited_registered_apps_path, "w") as file:
            json.dump(encoded, file)



import logging
from logging.handlers import RotatingFileHandler
from colorlog import ColoredFormatter


def configure_logging(log_file="app.log", log_level=logging.INFO):
    # Create a logger
    logger = logging.getLogger()
    logger.setLevel(log_level)

    # Create a file handler and set the log level
    file_handler = RotatingFileHandler(log_file, maxBytes=5 * 1024 * 1024, backupCount=2)
    file_handler.setLevel(log_level)

    # Create a formatter for the file handler (without colors)
    file_formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    file_handler.setFormatter(file_formatter)

    # Create a colored formatter for the console
    console_formatter = ColoredFormatter(
        "%(log_color)s%(levelname)-8s%(reset)s %(white)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            "DEBUG": "blue",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red",
        },
    )

    # Set the formatter for the console
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(console_formatter)

    # Add handlers to the logger
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)

    return logger


configure_logging()



import dataclasses
from typing import Optional

from streamlit_hub.models.RunningProcess import RunningProcess


@dataclasses.dataclass
class App:
    name: str
    run_by_default: bool
    running_process: Optional[RunningProcess] = None
    desired_port: Optional[str] = None


@dataclasses.dataclass
class LocalApp(App):
    path: str = ""


@dataclasses.dataclass
class RepoApp(App):
    repo_url: str = ""
    local_path: str = ""
    streamlit_entry_point_in_repo: str = ""
    branch: str = ""



import streamlit as st
import importlib
from streamlit_hub.client import register, status
import streamlit_hub.manager.manager
from streamlit_hub.manager.manager import Manager

importlib.reload(streamlit_hub.manager.manager)

st.set_page_config(page_title="Streamlit Hub", page_icon="👋", layout="wide")

# Custom CSS styles
custom_styles = """
<style>
</style>
"""

# Inject custom styles
st.markdown(custom_styles, unsafe_allow_html=True)


@st.cache_resource()
def get_manager():
    return Manager()


def run():
    manager = get_manager()
    st.title("App Manager Control Panel")
    page = st.sidebar.radio("Select Page", ["Home", "Register App"])
    if page == "Home":
        status.show_status(manager)
    else:
        register.show_register(manager)

    st.markdown(
        """
    **For more information:**
    - [Streamlit Documentation](https://docs.streamlit.io/)
    - [GitHub Repository](https://github.com/galaplexus/streamlit-hub)
    """
    )


if __name__ == "__main__":
    run()
